package celular.palavras;

/**
  * Universidade Federal de Pernambuco
  * Centro de Informtica
  * Disciplina: Introducao a Programao
  * Projeto Celular
  *
  * Tipo: ListaPalavras
  *
  * @author: Alberto Csar C. Frana     (accf2@cin.ufpe.br)
  * @author: Luciana Pereira Oliveira    (lpo@cin.ufpe.br)
  *
  * @since 2002
  * @version 1.0
  **/

import celular.util.Console;

public class ListaDePalavras {

  private Palavra[] elementos;
  private int proxima;  //indica o proximo indice que pode receber um objeto
  private int maximo;  //indica o tamanho maximo do array de objetos

  private final static byte NAO_EXISTE = -1;
  private final static int TAMANHO_MAXIMO_INICIAL = 5;

  private int indiceParaIteracao;

 /**
   * Construtor da classe
   *
   */
  public ListaDePalavras() {
     this.reset();
     maximo = TAMANHO_MAXIMO_INICIAL;
     elementos = new Palavra[maximo];
     proxima = 0;
  }

    /**
     * Mtodo retorna o valor do atributo proxima
     *
     * @return  Um inteiro que representa o valor do atributo proxima
     *
     */
    public int getTamanho() {
    	return proxima;
    }

   /**
     * Mtodo exclui uma palavra
     *
     * @param objeto do tipo Palavra  (recebe uma palavra)
     *
     */
    public void excluir(Palavra palavra) {
      if (palavra != null){
        int i = this.posicao(palavra);
        if (i == NAO_EXISTE) {
           new Console().println("ERRO: A palavra nao existe!!");
        } else {
            elementos[i] = null;
            elementos[i] = elementos[proxima - 1];
            elementos[proxima - 1] = null;
            proxima = proxima - 1;
            new Console().println("!!A palavra foi removida com sucesso!!");
        }
      }
    }

   /**
    * Mtodo que duplica o tamanho do array quando necessario
    *
    *
    */
   private void duplicarTamanho() {
     Palavra[] temp = new Palavra[this.maximo * 2];
     for (int i = 0; i < this.maximo; i = i + 1) {
       temp[i] = this.elementos[i];
     }
     this.elementos = temp;
     this.maximo = this.maximo * 2;
   }

  /**
    * Mtodo que insere um objeto no array
    *
    * @param objeto do tipo Palavra  (recebe uma palavra)
    *
    */
   public void inserir(Palavra palavra) {
     if (palavra != null){
          if (this.proxima == this.maximo) {
              this.duplicarTamanho();
          }
          this.elementos[proxima] = palavra;
          this.proxima = this.proxima + 1;
      } else {
          new Console().println("ERRO: a palavra nao eh valida!");
    }
   }

   /** Metodo retorna todas as palavras do dicionario
   *   que possuem a letra da posicao indicada igual a
   *   alguma letra passada como parametro
   *   @param  letra1         do tipo char
   *   @param  letra2         do tipo char
   *   @param  letra3         do tipo char
   *   @param  posicao        do tipo int
   *   @return lista          do tipo Lista
   *
   **/
  public ListaDePalavras filtrar(char[] letras,int posicaoAtual){
    ListaDePalavras temp = new ListaDePalavras();
    Palavra palavra;
    for (int i = 0 ; i < proxima; i++){
      palavra = elementos[i];
      if(palavra.temLetras(letras,posicaoAtual)){
        temp.inserir(palavra);
      }
    }
    return temp;
  }


  /**
    * Mtodo que indica existe String passada como parametro no array
    *
    * @param  chave   do tipo String
    * @return boolean informando se existe String procurada
    *
    */
  public boolean existe(Palavra chave) {
     boolean resposta = false;
     if (chave != null){
         if (this.posicao(chave) == NAO_EXISTE) {
            resposta = false;
         } else {
    	    resposta = true;
         }
     }
     return resposta;
  }


  /**
     * Mtodo que procura String na lista de array
     *
     * @param chave   do tipo Palavra
     * @return i       do tipo int
     *
     */
  private int posicao(Palavra chave) {
    int i = 0;
    boolean achou = false;
    if (chave != null){
        while(!achou && i < proxima) {
          if (chave.equals(this.elementos[i])) {
                   achou = true;
          } else {
              i = i + 1;
          }
        }
        if (!achou) {
           i = NAO_EXISTE;
        }
    } else {
        i = NAO_EXISTE;
    }

    return i;
  }


/**
  *  Mtodo que reseta o proxima de navegacao dentro da lista
  */
  public void reset(){
      this.indiceParaIteracao = 0;
  }

/**
  * Mtodo que retorna a proxima palavra da lista de acordo com o proxima de
  * navegao dentro da lista
  *
  *
  */
  public Palavra proxima(){
    Palavra retorno = null;
    if (indiceParaIteracao < proxima) {
      retorno = elementos[indiceParaIteracao];
      indiceParaIteracao = indiceParaIteracao + 1;
    }
    return retorno;
  }

  //main para verificacao de alguns metodos
  public static void main(String args[]) {
    Console c = new Console();
    Palavra p1,p2,p3,p4,p6,p5;
    p1 = new Palavra("casa");
    ListaDePalavras lista = new ListaDePalavras();
    lista.inserir(p1);
    p2 = new Palavra("carro");
    lista.inserir(p2);
    p3 = new Palavra("estrada");
    lista.inserir(p3);
    p4 = new Palavra("onda");
    lista.inserir(p4);
    p5 = new Palavra("roda");
    lista.inserir(p5);
    p6 = new Palavra("praia");
    lista.inserir(p6);
    c.println("tam" +lista.getTamanho());
    c.println("get 0"+ (lista.proxima()).getPalavra());
    lista.excluir(p4);
    c.println("tam: "+lista.getTamanho());
    lista.reset();
    c.println("get 0!!!"+ (lista.proxima()).getPalavra());
    c.println("Verifica se existe casa "+lista.existe(new Palavra("casa")));
    c.println("Verifica se existe mm "+lista.existe(new Palavra("mm")));
    c.println("Verifica posica de casa "+lista.posicao(new Palavra("casa")));
  }
}
